@@ -31,8 +31,9 @@ import numpy
 from scipy.interpolate import interp1d
 from scipy import special
 
-from pycbc.waveform.spa_tmplt import spa_tmplt
+from pycbc.waveform import get_fd_waveform_sequence
 from pycbc.detector import Detector
+from pycbc.types import Array
 
 from .gaussian_noise import BaseGaussianNoise
 
@@ -81,14 +82,15 @@ def setup_bins(f_full, f_lo, f_hi, chi=1.0, eps=0.5):
     # frequency grid points
     fbin = dphi2f(dphi_grid)
     # indices of frequency grid points in the FFT array
-    fbin_ind = numpy.array([numpy.argmin(numpy.absolute(f_full - ff)) for
-                            ff in fbin])
+    fbin_ind = numpy.unique([numpy.argmin(numpy.absolute(f_full - ff)) for
+                             ff in fbin])
     # make sure grid points are precise
     fbin = numpy.array([f_full[i] for i in fbin_ind])
+    nbin = len(fbin)
     return nbin, fbin, fbin_ind
 
 
-class RelativeSPA(BaseGaussianNoise):
+class Relative(BaseGaussianNoise):
     r"""Model that assumes the likelihood in a region around the peak
     is slowly varying such that a linear approximation can be made, and
     likelihoods can be calculated at a coarser frequency resolution. For
@@ -140,14 +142,11 @@ class RelativeSPA(BaseGaussianNoise):
         All other keyword arguments are passed to
         :py:class:`BaseGaussianNoise`.
     """
-    name = "relative_spa"
+    name = "relative"
 
     def __init__(self, variable_params, data, low_frequency_cutoff,
-                 mass1_ref, mass2_ref, spin1z_ref, spin2z_ref,
-                 ra_ref, dec_ref, tc_ref,
-                 epsilon=0.5,
-                 **kwargs):
-        super(RelativeSPA, self).__init__(
+                 fiducial_params=None, epsilon=0.5, **kwargs):
+        super(Relative, self).__init__(
             variable_params, data, low_frequency_cutoff, **kwargs)
         # check that all of the frequency cutoffs are the same
         # FIXME: this can probably be loosened at some point
@@ -168,32 +167,39 @@ class RelativeSPA(BaseGaussianNoise):
         self.comp_data = {ifo: d.numpy() for ifo, d in self.data.items()}
         self.comp_psds = {ifo: p.numpy() for ifo, p in self.psds.items()}
         # store fiducial waveform params
-        self.mass1_ref = float(mass1_ref)
-        self.mass2_ref = float(mass2_ref)
-        self.spin1z_ref = float(spin1z_ref)
-        self.spin2z_ref = float(spin2z_ref)
-        self.ra_ref = float(ra_ref)
-        self.dec_ref = float(dec_ref)
-        self.tc_ref = float(tc_ref)
+        self.fid_params = fiducial_params
 
         # get detector-specific arrival times relative to end of data
         dt = {ifo:
-              self.det[ifo].time_delay_from_earth_center(self.ra_ref,
-                                                         self.dec_ref,
-                                                         self.tc_ref)
+              self.det[ifo].time_delay_from_earth_center(
+                  self.fid_params['ra'], self.fid_params['dec'],
+                  self.fid_params['tc'])
               for ifo in self.data}
-        self.ta = {ifo: self.tc_ref + dt[ifo] - self.end_time
+        self.ta = {ifo: self.fid_params['tc'] + dt[ifo] - self.end_time
                    for ifo in self.data}
 
         # generate fiducial waveform
-        logging.info("Generating fiducial waveform")
-        hp = spa_tmplt(f_lower=kmins[0]*self.df, f_upper=(kmaxs[0]+1)*self.df,
-                       delta_f=self.df, mass1=self.mass1_ref,
-                       mass2=self.mass2_ref, spin1z=self.spin1z_ref,
-                       spin2z=self.spin2z_ref, distance=1.,
-                       spin_order=-1, phase_order=-1)
-        hp.resize(len(self.f))
-        self.h00 = numpy.array(hp)
+        f_lo = kmins[0] * self.df
+        f_hi = kmaxs[0] * self.df
+        logging.info("Generating fiducial waveform from %s to %s Hz",
+                     f_lo, f_hi)
+        # prune low frequency samples to avoid waveform errors
+        nbelow = sum(self.f < 10)
+        fpoints = Array(self.f.astype(numpy.float64))[nbelow:]
+        approx = self.static_params['approximant']
+        fid_hp, fid_hc = get_fd_waveform_sequence(approximant=approx,
+                                                  sample_points=fpoints,
+                                                  **self.fid_params)
+        self.h00 = {}
+        for ifo in self.data:
+            # make copy of fiducial wfs, adding back in low frequencies
+            hp0 = numpy.concatenate([[0j] * nbelow, fid_hp.copy()])
+            hc0 = numpy.concatenate([[0j] * nbelow, fid_hc.copy()])
+            fp, fc = self.det[ifo].antenna_pattern(
+                self.fid_params['ra'], self.fid_params['dec'],
+                self.fid_params['polarization'], self.fid_params['tc'])
+            tshift = numpy.exp(-2.0j * numpy.pi * self.f * self.ta[ifo])
+            self.h00[ifo] = numpy.array(hp0 * fp + hc0 * fc) * tshift
 
         # compute frequency bins
         logging.info("Computing frequency bins")
@@ -203,13 +209,14 @@ class RelativeSPA(BaseGaussianNoise):
         logging.info("Using %s bins for this model", nbin)
         # store bins and edges in sample and frequency space
         self.edges = fbin_ind
-        self.fedges = numpy.array(fbin).astype(numpy.float32)
+        self.fedges = numpy.array(fbin).astype(numpy.float64)
         self.bins = numpy.array([(self.edges[i], self.edges[i+1]) for
                                  i in range(len(self.edges) - 1)])
         self.fbins = numpy.array([(fbin[i], fbin[i+1]) for
                                   i in range(len(fbin) - 1)])
         # store low res copy of fiducial waveform
-        self.h00_sparse = self.h00.copy().take(self.edges)
+        self.h00_sparse = {ifo: self.h00[ifo].copy().take(self.edges) for ifo
+                           in self.h00}
 
         # compute summary data
         logging.info("Calculating summary data at frequency resolution %s Hz",
@@ -227,53 +234,30 @@ class RelativeSPA(BaseGaussianNoise):
             containing bin coefficients a0, b0, a1, b1, for each frequency
             bin.
         """
-        # timeshift the fiducial waveform for each detector
-        shift = {ifo: numpy.exp(-2.0j * numpy.pi * self.f * self.ta[ifo]) for
-                 ifo in self.data}
-        h0 = {ifo: self.h00.copy() * shift[ifo] for ifo in self.data}
         # calculate coefficients
         sdat = {}
         for ifo in self.data:
-            hd = numpy.conjugate(self.comp_data[ifo]) * h0[ifo]
+            hd = numpy.conjugate(self.comp_data[ifo]) * self.h00[ifo]
             hd /= self.comp_psds[ifo]
-            hh = (numpy.absolute(h0[ifo]) ** 2.0) / self.comp_psds[ifo]
+            hh = (numpy.absolute(self.h00[ifo]) ** 2.0) / self.comp_psds[ifo]
             # constant terms
             a0 = numpy.array([4. * self.df * numpy.sum(hd[l:h]) for
                               l, h in self.bins])
             b0 = numpy.array([4. * self.df * numpy.sum(hh[l:h]) for
                               l, h in self.bins])
             # linear terms
-            bin_centers = [0.5 * (fl + fh) for fl, fh in self.fbins]
+            bin_lefts = [fl for fl, fh in self.fbins]
             a1 = numpy.array([4. * self.df
-                              * numpy.sum(hd[l:h] * (self.f[l:h] - bc)) for
-                              (l, h), bc in zip(self.bins, bin_centers)])
+                              * numpy.sum(hd[l:h] * (self.f[l:h] - bl)) for
+                              (l, h), bl in zip(self.bins, bin_lefts)])
             b1 = numpy.array([4. * self.df
-                              * numpy.sum(hh[l:h] * (self.f[l:h] - bc)) for
-                              (l, h), bc in zip(self.bins, bin_centers)])
+                              * numpy.sum(hh[l:h] * (self.f[l:h] - bl)) for
+                              (l, h), bl in zip(self.bins, bin_lefts)])
 
             sdat[ifo] = {'a0': a0, 'a1': a1,
                          'b0': b0, 'b1': b1}
         return sdat
 
-    def waveform_ratio(self, p, htf, dtc=0.0):
-        """Calculate waveform ratio between template and fiducial
-        waveforms.
-        """
-        # generate template
-        hp = spa_tmplt(sample_points=self.fedges,
-                       mass1=p['mass1'], mass2=p['mass2'],
-                       spin1z=p['spin1z'], spin2z=p['spin2z'],
-                       distance=1., spin_order=-1, phase_order=-1)
-        htarget = numpy.array(hp)
-        # apply antenna pattern, inclination, and distance factor
-        htarget *= htf
-        # compute waveform ratio and timeshift
-        shift = numpy.exp(-2.0j * numpy.pi * self.fedges * dtc)
-        r = htarget / self.h00_sparse * shift
-        r0 = 0.5 * (r[:-1] + r[1:])
-        r1 = (r[1:] - r[:-1]) / (self.fedges[1:] - self.fedges[:-1])
-        return numpy.array([r0, r1], dtype=numpy.complex128)
-
     def _loglr(self):
         r"""Computes the log likelihood ratio,
 
@@ -294,32 +278,35 @@ class RelativeSPA(BaseGaussianNoise):
         p = self.current_params.copy()
         p.update(self.static_params)
 
-        llr = 0.
+        hh = 0.
+        hd = 0j
         for ifo in self.data:
             # get detector antenna pattern
             fp, fc = self.det[ifo].antenna_pattern(p['ra'], p['dec'],
                                                    p['polarization'],
                                                    p['tc'])
-            ip = numpy.cos(p['inclination'])
-            ic = 0.5 * (1.0 + ip * ip)
-            htf = (fp * ip + 1.0j * fc * ic) / p['distance']
-            # get timeshift relative to fiducial waveform
+            # get timeshift relative to end of data
             dt = self.det[ifo].time_delay_from_earth_center(p['ra'], p['dec'],
                                                             p['tc'])
-            dtc = p['tc'] + dt - self.end_time - self.ta[ifo]
+            dtc = p['tc'] + dt - self.end_time
+            tshift = numpy.exp(-2.0j * numpy.pi * self.fedges * dtc)
             # generate template and calculate waveform ratio
-            r0, r1 = self.waveform_ratio(p, htf, dtc=dtc)
-            # <h, d> is real part of sum over bins of A0r0 + A1r1
-            hd = numpy.sum(self.sdat[ifo]['a0'] * r0
-                           + self.sdat[ifo]['a1'] * r1).real
-            # marginalize over phase
-            hd = numpy.log(special.i0e(hd)) + abs(hd)
-            # <h, h> is real part of sum over bins of B0|r0|^2 + 2B1Re(r1r0*)
-            hh = numpy.sum(self.sdat[ifo]['b0'] * numpy.absolute(r0) ** 2.
-                           + 2. * self.sdat[ifo]['b1']
-                           * (r1 * numpy.conjugate(r0)).real).real
-            # increment loglr
-            llr += (hd - 0.5 * hh)
+            hp, hc = get_fd_waveform_sequence(sample_points=Array(self.fedges),
+                                              **p)
+            htilde = numpy.array(fp * hp + fc * hc) * tshift
+            r = (htilde / self.h00_sparse[ifo]).astype(numpy.complex128)
+            r0 = r[:-1]
+            r1 = (r[1:] - r[:-1]) / (self.fedges[1:] - self.fedges[:-1])
+
+            # <h, d> is sum over bins of A0r0 + A1r1
+            hd += numpy.sum(self.sdat[ifo]['a0'] * r0
+                            + self.sdat[ifo]['a1'] * r1)
+            # <h, h> is sum over bins of B0|r0|^2 + 2B1Re(r1r0*)
+            hh += numpy.sum(self.sdat[ifo]['b0'] * numpy.absolute(r0) ** 2.
+                            + 2. * self.sdat[ifo]['b1']
+                            * (r1 * numpy.conjugate(r0)).real)
+        hd = abs(hd)
+        llr = numpy.log(special.i0e(hd)) + hd - 0.5 * hh
         return float(llr)
 
     def write_metadata(self, fp):
@@ -330,12 +317,27 @@ class RelativeSPA(BaseGaussianNoise):
         fp : pycbc.inference.io.BaseInferenceFile instance
             The inference file to write to.
         """
-        super(RelativeSPA, self).write_metadata(fp)
-        fp.attrs['mass1_ref'] = self.mass1_ref
-        fp.attrs['mass2_ref'] = self.mass2_ref
-        fp.attrs['spin1z_ref'] = self.spin1z_ref
-        fp.attrs['spin2z_ref'] = self.spin2z_ref
-        fp.attrs['ra_ref'] = self.ra_ref
-        fp.attrs['dec_ref'] = self.dec_ref
-        fp.attrs['tc_ref'] = self.tc_ref
+        super(Relative, self).write_metadata(fp)
         fp.attrs['epsilon'] = self.epsilon
+        for p, v in self.fid_params.items():
+            fp.attrs['{}_ref'.format(p)] = v
+
+    @staticmethod
+    def extra_args_from_config(cp, section, skip_args=None, dtypes=None):
+        """Adds reading fiducial waveform parameters from config file.
+        """
+        # add fiducial params to skip list
+        skip_args += [option for option in cp.options(section) if
+                      option.endswith('_ref')]
+        args = super(Relative, Relative).extra_args_from_config(
+            cp, section, skip_args=skip_args, dtypes=dtypes)
+        # get fiducial params from config
+        fid_params = {p.replace('_ref', ''): float(cp.get('model', p)) for p
+                      in cp.options('model') if p.endswith('_ref')}
+        # add optional params with default values if not specified
+        opt_params = {'ra': numpy.pi, 'dec': 0.0, 'inclination': 0.0,
+                      'polarization': numpy.pi}
+        fid_params.update({p: opt_params[p] for p in opt_params if p
+                           not in fid_params})
+        args.update({'fiducial_params': fid_params})
+        return args
