@@ -36,7 +36,8 @@ import numpy
 
 from pycbc.workflow import ConfigParser
 from pycbc.filter import autocorrelation
-from pycbc.inference.io import validate_checkpoint_files
+from pycbc.inference.io import (validate_checkpoint_files, loadfile)
+from pycbc.inference.io.base_mcmc import nsamples_in_chain
 
 from .base import setup_output
 from .base import initial_dist_from_config
@@ -196,7 +197,7 @@ class BaseMCMC(object):
     ----------
     p0
     pos
-    nwalkers
+    nchains
     niterations
     checkpoint_interval
     checkpoint_signal
@@ -207,14 +208,16 @@ class BaseMCMC(object):
     thin_safety_factor
     burn_in
     effective_nsamples
-    acls
-    acts
+    acl
+    raw_acls
+    act
+    raw_acts
     """
     _lastclear = None  # the iteration when samples were cleared from memory
     _itercounter = None  # the number of iterations since the last clear
     _pos = None
     _p0 = None
-    _nwalkers = None
+    _nchains = None
     _burn_in = None
     _acls = None
     _checkpoint_interval = None
@@ -229,18 +232,24 @@ class BaseMCMC(object):
         """What shape the sampler's samples arrays are in, excluding
         the iterations dimension.
 
-        For example, if a sampler uses 20 walkers and 3 temperatures, this
+        For example, if a sampler uses 20 chains and 3 temperatures, this
         would be ``(3, 20)``. If a sampler only uses a single walker and no
         temperatures this would be ``()``.
         """
         pass
 
     @property
-    def nwalkers(self):
-        """The number of walkers used."""
-        if self._nwalkers is None:
-            raise ValueError("number of walkers not set")
-        return self._nwalkers
+    def nchains(self):
+        """The number of chains used."""
+        if self._nchains is None:
+            raise ValueError("number of chains not set")
+        return self._nchains
+
+    @nchains.setter
+    def nchains(self, value):
+        """Sets the number of chains."""
+        # we'll actually store it to the nchains attribute
+        self._nchains = int(value)
 
     @property
     def niterations(self):
@@ -311,7 +320,7 @@ class BaseMCMC(object):
             # effective samples
             if self.target_eff_nsamples is not None:
                 target_samps_per_chain = int(numpy.ceil(
-                    self.target_eff_nsamples / self.nwalkers))
+                    self.target_eff_nsamples / self.nchains))
                 if n <= target_samps_per_chain:
                     raise ValueError("max samples per chain must be > target "
                                      "effective number of samples per walker "
@@ -380,7 +389,7 @@ class BaseMCMC(object):
 
     @property
     def p0(self):
-        """A dictionary of the initial position of the walkers.
+        """A dictionary of the initial position of the chains.
 
         This is set by using ``set_p0``. If not set yet, a ``ValueError`` is
         raised when the attribute is accessed.
@@ -393,7 +402,7 @@ class BaseMCMC(object):
         return p0
 
     def set_p0(self, samples_file=None, prior=None):
-        """Sets the initial position of the walkers.
+        """Sets the initial position of the chains.
 
         Parameters
         ----------
@@ -478,9 +487,9 @@ class BaseMCMC(object):
                 nsamples = fp.effective_nsamples
         elif self.target_niterations is not None:
             # the number of samples is the number of iterations times the
-            # number of walkers
-            target_nsamples = self.nwalkers * self.target_niterations
-            nsamples = self._lastclear * self.nwalkers
+            # number of chains
+            target_nsamples = self.nchains * self.target_niterations
+            nsamples = self._lastclear * self.nchains
         else:
             raise ValueError("must set either target_eff_nsamples or "
                              "target_niterations; see set_target")
@@ -511,7 +520,7 @@ class BaseMCMC(object):
                 logging.info("Have {} effective samples post burn in".format(
                     nsamples))
             else:
-                nsamples += iterinterval * self.nwalkers
+                nsamples += iterinterval * self.nchains
 
     @property
     def burn_in(self):
@@ -522,24 +531,12 @@ class BaseMCMC(object):
         """Sets the object to use for doing burn-in tests."""
         self._burn_in = burn_in
 
-    @property
+    @abstractmethod
     def effective_nsamples(self):
         """The effective number of samples post burn-in that the sampler has
-        acquired so far."""
-        try:
-            act = numpy.array(list(self.acts.values())).max()
-        except (AttributeError, TypeError):
-            act = numpy.inf
-        if self.burn_in is None:
-            nperwalker = max(int(self.niterations // act), 1)
-        elif self.burn_in.is_burned_in:
-            nperwalker = int(
-                (self.niterations - self.burn_in.burn_in_iteration) // act)
-            # after burn in, we always have atleast 1 sample per walker
-            nperwalker = max(nperwalker, 1)
-        else:
-            nperwalker = 0
-        return self.nwalkers * nperwalker
+        acquired so far.
+        """
+        pass
 
     @abstractmethod
     def run_mcmc(self, niterations):
@@ -589,32 +586,25 @@ class BaseMCMC(object):
             logging.info("No samples written due to thinning")
         else:
             # check for burn in, compute the acls
-            self.acls = None
+            self.raw_acls = None
             if self.burn_in is not None:
                 logging.info("Updating burn in")
                 self.burn_in.evaluate(self.checkpoint_file)
-                burn_in_index = self.burn_in.burn_in_index
-                logging.info("Is burned in: %r", self.burn_in.is_burned_in)
-                if self.burn_in.is_burned_in:
-                    logging.info("Burn-in iteration: %i",
-                                 int(self.burn_in.burn_in_iteration))
-            else:
-                burn_in_index = 0
+                # write
+                for fn in [self.checkpoint_file, self.backup_file]:
+                    with self.io(fn, "a") as fp:
+                        self.burn_in.write(fp)
             # Compute acls; the burn_in test may have calculated an acl and
             # saved it, in which case we don't need to do it again.
-            if self.acls is None:
-                logging.info("Computing acls")
-                self.acls = self.compute_acl(self.checkpoint_file,
-                                             start_index=burn_in_index)
-            logging.info("ACT: %s",
-                         str(numpy.array(list(self.acts.values())).max()))
-            # write
+            if self.raw_acls is None:
+                logging.info("Computing autocorrelation time")
+                self.raw_acls = self.compute_acl(self.checkpoint_file)
+            # write acts, effective number of samples
             for fn in [self.checkpoint_file, self.backup_file]:
                 with self.io(fn, "a") as fp:
-                    if self.burn_in is not None:
-                        fp.write_burn_in(self.burn_in)
-                    if self.acls is not None:
-                        fp.write_acls(self.acls)
+                    if self.raw_acls is not None:
+                        fp.raw_acls = self.raw_acls
+                        fp.acl = self.acl
                     # write effective number of samples
                     fp.write_effective_nsamples(self.effective_nsamples)
         # check validity
@@ -737,27 +727,57 @@ class BaseMCMC(object):
         self.max_samples_per_chain = max_samps_per_chain
 
     @property
-    def acls(self):
-        """The autocorrelation lengths of each parameter's thinned chain."""
+    def raw_acls(self):
+        """Dictionary of parameter names -> autocorrelation lengths.
+
+        Depending on the sampler, the ACLs may be an integer, or an arrray of
+        values per chain and/or per temperature.
+
+        Returns ``None`` if no ACLs have been calculated.
+        """
         return self._acls
 
-    @acls.setter
-    def acls(self, acls):
-        """Sets the acls."""
+    @raw_acls.setter
+    def raw_acls(self, acls):
+        """Sets the raw acls."""
         self._acls = acls
 
-    @property
-    def acts(self):
-        """The autocorrelation times of each parameter.
+    @abstractmethod
+    def acl(self):
+        """The autocorrelation length.
 
-        The autocorrelation time is defined as the ACL times the
-        ``thin_interval``. It gives the number of iterations between
-        independent samples.
+        This method should convert the raw ACLs into an integer or array that
+        can be used to extract independent samples from a chain.
         """
-        if self.acls is None:
+        pass
+
+    @property
+    def raw_acts(self):
+        """Dictionary of parameter names -> autocorrelation time(s).
+
+        Returns ``None`` if no ACLs have been calculated.
+        """
+        acls = self.raw_acls
+        if acls is None:
             return None
         return {p: acl * self.thin_interval
-                for (p, acl) in self.acls.items()}
+                for (p, acl) in acls.items()}
+
+    @property
+    def act(self):
+        """The autocorrelation time(s).
+
+        The autocorrelation time is defined as the autocorrelation length times
+        the ``thin_interval``. It gives the number of iterations between
+        independent samples. Depending on the sampler, this may either be
+        a single integer or an array of values.
+
+        Returns ``None`` if no ACLs have been calculated.
+        """
+        acl = self.acl
+        if acl is None:
+            return None
+        return acl * self.thin_interval
 
     @abstractmethod
     def compute_acf(cls, filename, **kwargs):
@@ -772,120 +792,174 @@ class BaseMCMC(object):
         pass
 
 
-class MCMCAutocorrSupport(object):
-    """Provides class methods for calculating ensemble ACFs/ACLs.
+class EnsembleSupport(object):
+    """Adds support for ensemble MCMC samplers."""
+
+    @property
+    def nwalkers(self):
+        """The number of walkers used.
+
+        Alias of ``nchains``.
+        """
+        return self.nchains
+
+    @nwalkers.setter
+    def nwalkers(self, value):
+        """Sets the number of walkers."""
+        # we'll actually store it to the nchains attribute
+        self.nchains = value
+
+    @property
+    def acl(self):
+        """The autocorrelation length of the ensemble.
+
+        This is calculated by taking the maximum over all of the ``raw_acls``.
+        This works for both single and parallel-tempered ensemble samplers.
+
+        Returns ``None`` if no ACLs have been set.
+        """
+        acls = self.raw_acls
+        if acls is None:
+            return None
+        return numpy.array(list(acls.values())).max()
+
+    @property
+    def effective_nsamples(self):
+        """The effective number of samples post burn-in that the sampler has
+        acquired so far.
+        """
+        act = self.act
+        if act is None:
+            act = numpy.inf
+        if self.burn_in is None or not self.burn_in.is_burned_in:
+            start_iter = 0
+        else:
+            start_iter = self.burn_in.burn_in_iteration
+        nperwalker = nsamples_in_chain(start_iter, act, self.niterations)
+        if self.burn_in is not None and self.burn_in.is_burned_in:
+            # after burn in, we always have atleast 1 sample per walker
+            nperwalker = max(nperwalker, 1)
+        return int(self.nwalkers * nperwalker)
+
+
+#
+# =============================================================================
+#
+#              Functions for computing autocorrelation lengths
+#
+# =============================================================================
+#
+
+
+def ensemble_compute_acf(filename, start_index=None, end_index=None,
+                         per_walker=False, walkers=None, parameters=None):
+    """Computes the autocorrleation function for an ensemble MCMC.
+
+    By default, parameter values are averaged over all walkers at each
+    iteration. The ACF is then calculated over the averaged chain. An
+    ACF per-walker will be returned instead if ``per_walker=True``.
+
+    Parameters
+    -----------
+    filename : str
+        Name of a samples file to compute ACFs for.
+    start_index : int, optional
+        The start index to compute the acl from. If None (the default), will
+        try to use the number of burn-in iterations in the file; otherwise,
+        will start at the first sample.
+    end_index : int, optional
+        The end index to compute the acl to. If None (the default), will go to
+        the end of the current iteration.
+    per_walker : bool, optional
+        Return the ACF for each walker separately. Default is False.
+    walkers : int or array, optional
+        Calculate the ACF using only the given walkers. If None (the
+        default) all walkers will be used.
+    parameters : str or array, optional
+        Calculate the ACF for only the given parameters. If None (the
+        default) will calculate the ACF for all of the model params.
+
+    Returns
+    -------
+    dict :
+        Dictionary of arrays giving the ACFs for each parameter. If
+        ``per-walker`` is True, the arrays will have shape
+        ``nwalkers x niterations``.
     """
-
-    @classmethod
-    def compute_acf(cls, filename, start_index=None, end_index=None,
-                    per_walker=False, walkers=None, parameters=None):
-        """Computes the autocorrleation function of the model params in the
-        given file.
-
-        By default, parameter values are averaged over all walkers at each
-        iteration. The ACF is then calculated over the averaged chain. An
-        ACF per-walker will be returned instead if ``per_walker=True``.
-
-        Parameters
-        -----------
-        filename : str
-            Name of a samples file to compute ACFs for.
-        start_index : {None, int}
-            The start index to compute the acl from. If None, will try to use
-            the number of burn-in iterations in the file; otherwise, will start
-            at the first sample.
-        end_index : {None, int}
-            The end index to compute the acl to. If None, will go to the end
-            of the current iteration.
-        per_walker : optional, bool
-            Return the ACF for each walker separately. Default is False.
-        walkers : optional, int or array
-            Calculate the ACF using only the given walkers. If None (the
-            default) all walkers will be used.
-        parameters : optional, str or array
-            Calculate the ACF for only the given parameters. If None (the
-            default) will calculate the ACF for all of the model params.
-
-        Returns
-        -------
-        dict :
-            Dictionary of arrays giving the ACFs for each parameter. If
-            ``per-walker`` is True, the arrays will have shape
-            ``nwalkers x niterations``.
-        """
-        acfs = {}
-        with cls._io(filename, 'r') as fp:
-            if parameters is None:
-                parameters = fp.variable_params
-            if isinstance(parameters, string_types):
-                parameters = [parameters]
-            for param in parameters:
-                if per_walker:
-                    # just call myself with a single walker
-                    if walkers is None:
-                        walkers = numpy.arange(fp.nwalkers)
-                    arrays = [
-                        cls.compute_acf(filename, start_index=start_index,
-                                        end_index=end_index,
-                                        per_walker=False, walkers=ii,
-                                        parameters=param)[param]
-                        for ii in walkers]
-                    acfs[param] = numpy.vstack(arrays)
-                else:
-                    samples = fp.read_raw_samples(
-                        param, thin_start=start_index, thin_interval=1,
-                        thin_end=end_index, walkers=walkers,
-                        flatten=False)[param]
-                    samples = samples.mean(axis=0)
-                    acfs[param] = autocorrelation.calculate_acf(
-                        samples).numpy()
-        return acfs
-
-    @classmethod
-    def compute_acl(cls, filename, start_index=None, end_index=None,
-                    min_nsamples=10):
-        """Computes the autocorrleation length for all model params in the
-        given file.
-
-        Parameter values are averaged over all walkers at each iteration.
-        The ACL is then calculated over the averaged chain. If an ACL cannot
-        be calculated because there are not enough samples, it will be set
-        to ``inf``.
-
-        Parameters
-        -----------
-        filename : str
-            Name of a samples file to compute ACLs for.
-        start_index : int, optional
-            The start index to compute the acl from. If None, will try to use
-            the number of burn-in iterations in the file; otherwise, will start
-            at the first sample.
-        end_index : int, optional
-            The end index to compute the acl to. If None, will go to the end
-            of the current iteration.
-        min_nsamples : int, optional
-            Require a minimum number of samples to compute an ACL. If the
-            number of samples per walker is less than this, will just set to
-            ``inf``. Default is 10.
-
-        Returns
-        -------
-        dict
-            A dictionary giving the ACL for each parameter.
-        """
-        acls = {}
-        with cls._io(filename, 'r') as fp:
-            for param in fp.variable_params:
+    acfs = {}
+    with loadfile(filename, 'r') as fp:
+        if parameters is None:
+            parameters = fp.variable_params
+        if isinstance(parameters, string_types):
+            parameters = [parameters]
+        for param in parameters:
+            if per_walker:
+                # just call myself with a single walker
+                if walkers is None:
+                    walkers = numpy.arange(fp.nwalkers)
+                arrays = [
+                    ensemble_compute_acf(filename, start_index=start_index,
+                                         end_index=end_index,
+                                         per_walker=False, walkers=ii,
+                                         parameters=param)[param]
+                    for ii in walkers]
+                acfs[param] = numpy.vstack(arrays)
+            else:
                 samples = fp.read_raw_samples(
                     param, thin_start=start_index, thin_interval=1,
-                    thin_end=end_index, flatten=False)[param]
+                    thin_end=end_index, walkers=walkers,
+                    flatten=False)[param]
                 samples = samples.mean(axis=0)
-                # if < min number of samples, just set to inf
-                if samples.size < min_nsamples:
-                    acl = numpy.inf
-                else:
-                    acl = autocorrelation.calculate_acl(samples)
-                if acl <= 0:
-                    acl = numpy.inf
-                acls[param] = acl
-        return acls
+                acfs[param] = autocorrelation.calculate_acf(
+                    samples).numpy()
+    return acfs
+
+
+def ensemble_compute_acl(filename, start_index=None, end_index=None,
+                         min_nsamples=10):
+    """Computes the autocorrleation length for an ensemble MCMC.
+
+    Parameter values are averaged over all walkers at each iteration.
+    The ACL is then calculated over the averaged chain. If an ACL cannot
+    be calculated because there are not enough samples, it will be set
+    to ``inf``.
+
+    Parameters
+    -----------
+    filename : str
+        Name of a samples file to compute ACLs for.
+    start_index : int, optional
+        The start index to compute the acl from. If None, will try to use
+        the number of burn-in iterations in the file; otherwise, will start
+        at the first sample.
+    end_index : int, optional
+        The end index to compute the acl to. If None, will go to the end
+        of the current iteration.
+    min_nsamples : int, optional
+        Require a minimum number of samples to compute an ACL. If the
+        number of samples per walker is less than this, will just set to
+        ``inf``. Default is 10.
+
+    Returns
+    -------
+    dict
+        A dictionary giving the ACL for each parameter.
+    """
+    acls = {}
+    with loadfile(filename, 'r') as fp:
+        for param in fp.variable_params:
+            samples = fp.read_raw_samples(
+                param, thin_start=start_index, thin_interval=1,
+                thin_end=end_index, flatten=False)[param]
+            samples = samples.mean(axis=0)
+            # if < min number of samples, just set to inf
+            if samples.size < min_nsamples:
+                acl = numpy.inf
+            else:
+                acl = autocorrelation.calculate_acl(samples)
+            if acl <= 0:
+                acl = numpy.inf
+            acls[param] = acl
+        maxacl = numpy.array(list(acls.values())).max()
+        logging.info("ACT: %s", str(maxacl*fp.thin_interval))
+    return acls
