@@ -83,9 +83,9 @@ def sigma_cached(self, psd):
                 self.sigma_view = self[self.sslice].squared_norm() * 4.0 * self.delta_f
 
             if not hasattr(psd, 'invsqrt'):
-                psd.invsqrt = 1.0 / psd[self.sslice]
+                psd.invsqrt = 1.0 / psd
 
-            self._sigmasq[key] = self.sigma_view.inner(psd.invsqrt)
+            self._sigmasq[key] = self.sigma_view.inner(psd.invsqrt[self.sslice])
     return self._sigmasq[key]
 
 # dummy class needed for loading LIGOLW files
@@ -398,6 +398,8 @@ class TemplateBank(object):
         """ Return the end frequency of the waveform at the given index value
         """
         from pycbc.waveform.waveform import props
+        if hasattr(self.table[index], 'f_final'):
+            return self.table[index].f_final
 
         return pycbc.waveform.get_waveform_end_frequency(
                                 self.table[index],
@@ -581,10 +583,13 @@ class LiveFilterBank(TemplateBank):
         # include ringdown) and the duration up to merger since they will be
         # erased by the type conversion below.
         ttotal = template_duration = -1
+        time_offset = None
         if hasattr(htilde, 'length_in_time'):
             ttotal = htilde.length_in_time
         if hasattr(htilde, 'chirp_length'):
             template_duration = htilde.chirp_length
+        if hasattr(htilde, 'time_offset'):
+            time_offset = htilde.time_offset
 
         self.table[index].template_duration = template_duration
 
@@ -598,6 +603,9 @@ class LiveFilterBank(TemplateBank):
         htilde.approximant = approximant
         htilde.end_frequency = f_end
 
+        if time_offset:
+            htilde.time_offset = time_offset
+
         # Add sigmasq as a method of this instance
         htilde.sigmasq = types.MethodType(sigma_cached, htilde)
 
@@ -721,15 +729,10 @@ class FilterBank(TemplateBank):
             f_end = (self.filter_length-1) * self.delta_f
 
         # Find the start frequency, if variable
-        if self.f_lower is None:
-            f_low = self.table[index].f_lower
-        elif self.max_template_length is not None:
-            f_low = find_variable_start_frequency(approximant,
-                                                  self.table[index],
-                                                  self.f_lower,
-                                                  self.max_template_length)
-        else:
-            f_low = self.f_lower
+        f_low = find_variable_start_frequency(approximant,
+                                              self.table[index],
+                                              self.f_lower,
+                                              self.max_template_length)
         logging.info('%s: generating %s from %s Hz' % (index, approximant, f_low))
 
         # Clear the storage memory
@@ -779,12 +782,17 @@ def find_variable_start_frequency(approximant, parameters, f_start, max_length,
     """ Find a frequency value above the starting frequency that results in a
     waveform shorter than max_length.
     """
-    l = max_length + 1
-    f = f_start - delta_f
-    while l > max_length:
-        f += delta_f
-        l = pycbc.waveform.get_waveform_filter_length_in_time(approximant,
-                                                      parameters, f_lower=f)
+    if (f_start is None):
+        f = parameters.f_lower
+    elif (max_length is not None):
+        l = max_length + 1
+        f = f_start - delta_f
+        while l > max_length:
+            f += delta_f
+            l = pycbc.waveform.get_waveform_filter_length_in_time(approximant,
+                                                          parameters, f_lower=f)
+    else :
+        f = f_start
     return f
 
 
@@ -828,14 +836,10 @@ class FilterBankSkyMax(TemplateBank):
             f_end = (self.filter_length-1) * self.delta_f
 
         # Find the start frequency, if variable
-        if self.max_template_length is not None:
-            f_low = find_variable_start_frequency(approximant,
-                                                  self.table[index],
-                                                  self.f_lower,
-                                                  self.max_template_length)
-        else:
-            f_low = self.f_lower
-
+        f_low = find_variable_start_frequency(approximant,
+                                              self.table[index],
+                                              self.f_lower,
+                                              self.max_template_length)
         logging.info('%s: generating %s from %s Hz', index, approximant, f_low)
 
         # What does this do???
